@@ -268,6 +268,7 @@ let __init_install_bindings= !Data.install_bindings
 let __init_format_metavariables = !Data.format_metavariables
 let __init_format_list_metavariables = !Data.format_list_metavariables
 let __init_constraint_scripts = !Data.constraint_scripts
+let __init_fresh_id_scripts = !Data.fresh_id_scripts
 let __init_sgrep_mode = !Flag_parsing_cocci.sgrep_mode
 let __init_show_SP = !Flag_parsing_cocci.show_SP
 let __init_show_iso_failures = !Flag_parsing_cocci.show_iso_failures
@@ -577,6 +578,7 @@ let coccinelle arglist =
   Data.format_metavariables := __init_format_metavariables;
   Data.format_list_metavariables := __init_format_list_metavariables;
   Data.constraint_scripts := __init_constraint_scripts;
+  Data.fresh_id_scripts := __init_fresh_id_scripts;
   Flag_parsing_cocci.sgrep_mode := __init_sgrep_mode;
   Flag_parsing_cocci.show_SP := __init_show_SP;
   Flag_parsing_cocci.show_iso_failures := __init_show_iso_failures;
