@@ -62,8 +62,8 @@ let dumb_astcocci_decl = function
    A.UnInit (stg, typa, sa, attr, _)     -> ()
  | A.Init (stg, typa, sa, attr, _, expa, _) -> ()
  | A.FunProto _ -> ()
- | A.TyDecl (typa, _)     -> ()
- | A.MacroDecl(stg, fn, _, eas, _, _) -> ()
+ | A.TyDecl (typa, _, _)     -> ()
+ | A.MacroDecl(stg, fn, _, eas, _, _, _) -> ()
  | A.MacroDeclInit(stg, fn, _, eas, _, _, _, _) -> ()
  | A.MetaDecl _ -> ()
  | A.AsDecl _ -> ()
@@ -76,9 +76,9 @@ let dumb_astcocci_initialiser = function (* seems same as the above *)
     A.Init(stg,ty,id,attr,eq,ini,sem) -> ()
   | A.UnInit(stg,ty,id,attr,sem) -> ()
   | A.FunProto _ -> ()
-  | A.MacroDecl(_, fn, _, eas, _, _) -> ()
+  | A.MacroDecl(_, fn, _, eas, _, _, _) -> ()
   | A.MacroDeclInit(_, fn, _, eas, _, _, _, _) -> ()
-  | A.TyDecl(ty,sem) -> ()
+  | A.TyDecl(ty,attr,sem) -> ()
   | A.Typedef(d,ty1,ty2,pv) -> ()
   | A.DisjDecl(decls) -> ()
   | A.ConjDecl(decls) -> ()
@@ -87,12 +87,16 @@ let dumb_astcocci_initialiser = function (* seems same as the above *)
   | A.OptDecl(decl) -> ()
 
 let dumb_astcocci_field = function
-   A.Field (typa, sa, _bf, _)     -> ()
- | A.MetaField _ -> ()
- | A.MetaFieldList _ -> ()
- | A.DisjField xs -> ()
- | A.ConjField xs -> ()
- | A.OptField _ -> ()
+    A.Field (typa, sa, _bf, _)     -> ()
+  | A.MetaField _ -> ()
+  | A.MetaFieldList _ -> ()
+
+let dumb_astcocci_ann_field = function
+  | A.FElem(bef,allminus,fld) -> ()
+  | A.Fdots(dots,whencode) -> ()
+  | A.DisjField xs -> ()
+  | A.ConjField xs -> ()
+  | A.OptField _ -> ()
 
 let dumb_astcocci_expr = function
  | A.MetaExpr (ida,_,_, opttypa, _, _, _bitfield) -> ()
@@ -101,12 +105,12 @@ let dumb_astcocci_expr = function
  | A.Edots (_,_) -> ()
  | A.MetaErr _ -> ()
  | A.Ident ida -> ()
- | A.Constant (A.String sa,_,_,_) -> ()
- | A.Constant (A.Char sa,_,_,_) -> ()
+ | A.Constant (A.String(sa,_),_,_,_) -> ()
+ | A.Constant (A.Char(sa,_),_,_,_) -> ()
  | A.Constant (A.Int sa,_,_,_) -> ()
  | A.Constant (A.Float sa,_,_,_) -> ()
  | A.Constant (A.DecimalConst _,_,_,_) -> ()
- | A.StringConstant (lq,frags,rq) -> ()
+ | A.StringConstant (lq,frags,rq,sz) -> ()
  | A.FunCall (ea1, _, eas, _) -> ()
  | A.Assignment (ea1, opa, ea2, _) -> ()
  | A.Sequence (ea1, opa, ea2) -> ()
@@ -119,7 +123,7 @@ let dumb_astcocci_expr = function
  | A.ArrayAccess (ea1, _, ea2, _) -> ()
  | A.RecordAccess (ea, _, ida) -> ()
  | A.RecordPtAccess (ea, _, ida) -> ()
- | A.Cast (_, typa, _, ea) -> ()
+ | A.Cast (_, typa, attra, _, ea) -> ()
  | A.SizeOfExpr (_, ea) -> ()
  | A.SizeOfType (_, _, typa, _) -> ()
  | A.TypeExp (typa) -> ()
@@ -144,7 +148,8 @@ let dumb_astcocci_type = function
  | A.BaseType (basea,strings) -> ()
  | A.SignedT (signa,tya) -> ()
  | A.Pointer (typa, _) -> ()
- | A.FunctionPointer(ty,lp1,star,rp1,lp2,params,rp2) -> ()
+ | A.ParenType(lp,ty,rp) -> ()
+ | A.FunctionType(ty,lp,params,rp) -> ()
  | A.Array (typa, _, eaopt, _) -> ()
  | A.Decimal(_, _, _, _, _, _) -> ()
  | A.EnumName(en, ena) -> ()
@@ -154,6 +159,7 @@ let dumb_astcocci_type = function
  | A.TypeOfExpr(tf, lp, exp, rp) -> ()
  | A.TypeOfType(tf, lp, ty, rp) -> ()
  | A.TypeName sa -> ()
+ | A.AutoType _ -> ()
 
 
 (* ------------------------------------------------------------------------- *)
