@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -248,6 +248,7 @@ let pred2c = function
   | Lib_engine.EscTrueBranch -> ("\\msf{EscTrueBranch}",13)
   | Lib_engine.FalseBranch -> ("\\msf{FalseBranch}",11)
   | Lib_engine.After -> ("\\msf{After}",5)
+  | Lib_engine.GotoAfter -> ("\\msf{GotoAfter}",9)
   | Lib_engine.FallThrough -> ("\\msf{FallThrough}",11)
   | Lib_engine.LoopFallThrough -> ("\\msf{LoopFallThrough}",15)
   | Lib_engine.Return -> ("\\msf{Return}",6)
