@@ -149,6 +149,8 @@ let rec is_completed_and_simplified ty =
 
   | TypeOfExpr e ->
       true (* well we don't handle it, so can't really say it's completed *)
+  | AutoType -> true (* this version is the most complete until we have
+                        inference over auto *)
 
 
 let is_completed_typedef_fullType x = raise Todo
@@ -302,6 +304,7 @@ let (fake_function_type:
                 { Ast_c.p_namei = None;
                   p_register = false, Ast_c.noii;
                   p_type = ft;
+                  p_attr = [];
                 }
               in
               Some (paramtype, ii)
