@@ -113,6 +113,7 @@ and metavar =
   | MetaPosDecl of arity * meta_name (* name *)
   | MetaComDecl of arity * meta_name (* name *)
   | MetaFmtDecl of arity * meta_name (* name *)
+  | MetaAttributeDecl of arity * meta_name (* name *)
   | MetaFragListDecl of arity * meta_name (* name *) * list_len (*len*)
   | MetaAnalysisDecl of string * meta_name (* name *)
   | MetaDeclarerDecl of arity * meta_name (* name *)
@@ -121,8 +122,14 @@ and metavar =
 
 and list_len = AnyLen | MetaLen of meta_name * constraints | CstLen of int
 
-and seed = NoVal | StringSeed of string | ListSeed of seed_elem list
+and seed = NoVal | StringSeed of string | ListSeed of seed_elem list | ScriptSeed of seed_script
 and seed_elem = SeedString of string | SeedId of meta_name
+and seed_script = (* similar to script_constraint but kept separated to allow more flexibility *)
+      string (* name of generated function *) *
+	string (* language *) *
+	(meta_name * metavar) list (* params *) *
+	script_position *
+	string (* code *)
 
 (* --------------------------------------------------------------------- *)
 (* --------------------------------------------------------------------- *)
@@ -156,7 +163,7 @@ and base_expression =
     Ident          of ident
   | Constant       of constant mcode
   | StringConstant of string mcode (* quote *) * string_fragment dots *
-		      string mcode (* quote *)
+		      string mcode (* quote *) * isWchar
   | FunCall        of expression * string mcode (* ( *) *
                       expression dots * string mcode (* ) *)
   | Assignment     of expression * assignOp * expression * bool
@@ -172,8 +179,8 @@ and base_expression =
 	              string mcode (* ] *)
   | RecordAccess   of expression * string mcode (* . *) * ident
   | RecordPtAccess of expression * string mcode (* -> *) * ident
-  | Cast           of string mcode (* ( *) * fullType * string mcode (* ) *) *
-                      expression
+  | Cast           of string mcode (* ( *) * fullType * attr list *
+                      string mcode (* ) *) * expression
 
   | SizeOfExpr     of string mcode (* sizeof *) * expression
   | SizeOfType     of string mcode (* sizeof *) * string mcode (* ( *) *
@@ -294,11 +301,12 @@ and  arithOp =
 and  logicalOp = Inf | Sup | InfEq | SupEq | Eq | NotEq | AndLog | OrLog
 
 and constant =
-    String of string
-  | Char   of string
+    String of string * isWchar
+  | Char   of string * isWchar
   | Int    of string
   | Float  of string
   | DecimalConst of (string * string * string)
+and isWchar = IsWchar | IsUchar | Isuchar | Isu8char | IsChar
 
 (* --------------------------------------------------------------------- *)
 (* Types *)
@@ -315,9 +323,9 @@ and base_typeC =
     BaseType        of baseType * string mcode list (* Yoann style *)
   | SignedT         of sign mcode * typeC option
   | Pointer         of fullType * string mcode (* * *)
-  | FunctionPointer of fullType *
-	          string mcode(* ( *)*string mcode(* * *)*string mcode(* ) *)*
-                  string mcode (* ( *)*parameter_list*string mcode(* ) *)
+  | ParenType       of string mcode (* ( *) * fullType * string mcode (* ) *)
+  | FunctionType    of fullType *
+                  string mcode (* ( *) * parameter_list * string mcode (* ) *)
   | Array           of fullType * string mcode (* [ *) *
 	               expression option * string mcode (* ] *)
   | Decimal         of string mcode (* decimal *) * string mcode (* ( *) *
@@ -326,7 +334,7 @@ and base_typeC =
 	               string mcode (* ) *) (* IBM C only *)
   | EnumName        of string mcode (*enum*) * ident option (* name *)
   | EnumDef  of fullType (* either EnumName or metavar *) *
-	string mcode (* { *) * expression dots * string mcode (* } *)
+	string mcode (* { *) * enum_decl dots * string mcode (* } *)
   | StructUnionName of structUnion mcode * ident option (* name *)
   | StructUnionDef  of fullType (* either StructUnionName or metavar *) *
 	string mcode (* { *) * annotated_field dots * string mcode (* } *)
@@ -335,6 +343,7 @@ and base_typeC =
   | TypeOfType      of string mcode (* typeof *) * string mcode (* ( *) *
                        fullType * string mcode (* ) *)
   | TypeName        of string mcode
+  | AutoType        of string mcode (* auto *) (* c++ >= 11 *)
 
   | MetaType        of meta_name mcode * constraints * keep_binding *
 	inherited
@@ -370,10 +379,11 @@ and base_declaration =
 	string mcode (* ( *) * parameter_list *
 	(string mcode (* , *) * string mcode (* ...... *) ) option *
 	string mcode (* ) *) * string mcode (* ; *)
-  | TyDecl of fullType * string mcode (* ; *)
+  | TyDecl of fullType * attr list * string mcode (* ; *)
   | MacroDecl of storage mcode option *
 	ident (* name *) * string mcode (* ( *) *
-        expression dots * string mcode (* ) *) * string mcode (* ; *)
+        expression dots * string mcode (* ) *) *
+        attr list * string mcode (* ; *)
   | MacroDeclInit of storage mcode option *
 	ident (* name *) * string mcode (* ( *) *
         expression dots * string mcode (* ) *) * string mcode (*=*) *
@@ -399,9 +409,6 @@ and annotated_decl = base_annotated_decl wrap
 
 and base_field =
     Field of fullType * ident option * bitfield option * string mcode (* ; *)
-  | DisjField of field list
-  | ConjField of field list
-  | OptField of field
   | MetaField of meta_name mcode * constraints * keep_binding * inherited
   | MetaFieldList of meta_name mcode * listlen * constraints * keep_binding *
 	inherited
@@ -413,10 +420,21 @@ and field = base_field wrap
 and base_annotated_field =
     FElem of mcodekind (* before the decl *) * bool (* true if all minus *) *
       field
-  | Fdots    of string mcode (* ... *) * field option (* whencode *)
+  | Fdots     of string mcode (* ... *) * field option (* whencode *)
+  | DisjField of annotated_field list
+  | ConjField of annotated_field list
+  | OptField  of annotated_field
 
 and annotated_field = base_annotated_field wrap
 
+and base_enum_decl =
+    Enum of ident * (string mcode (* = *) * expression) option
+  | EnumComma of string mcode (* , *)
+  | EnumDots of string mcode (* ... *) * enum_decl option (* whencode *)
+
+and enum_decl = base_enum_decl wrap
+
+
 (* --------------------------------------------------------------------- *)
 (* Initializers *)
 
@@ -452,8 +470,8 @@ and initialiser = base_initialiser wrap
 (* Parameter *)
 
 and base_parameterTypeDef =
-    VoidParam     of fullType
-  | Param         of fullType * ident option
+    VoidParam     of fullType * attr list
+  | Param         of fullType * ident option * attr list
 
   | MetaParam     of meta_name mcode * constraints * keep_binding * inherited
   | MetaParamList of meta_name mcode * listlen * constraints * keep_binding *
@@ -587,7 +605,11 @@ and fninfo =
   | FInline of string mcode
   | FAttr of attr
 
-and attr = string mcode
+and base_attr =
+    Attribute of string mcode
+  | MetaAttribute of meta_name mcode * constraints * keep_binding * inherited
+
+and attr = base_attr wrap
 
 and metaStmtInfo =
     NotSequencible | SequencibleAfterDots of dots_whencode list | Sequencible
@@ -744,6 +766,7 @@ and anything =
   | LogicalOpTag        of logicalOp
   | DeclarationTag      of declaration
   | FieldTag            of field
+  | EnumDeclTag         of enum_decl
   | InitTag             of initialiser
   | StorageTag          of storage
   | IncFileTag          of inc_file
@@ -752,6 +775,7 @@ and anything =
   | ForInfoTag          of forinfo
   | CaseLineTag         of case_line
   | StringFragmentTag   of string_fragment
+  | AttributeTag        of attr
   | ConstVolTag         of const_vol
   | Token               of string * info option
   | Directive           of added_string list
@@ -761,6 +785,7 @@ and anything =
   | StmtDotsTag         of statement dots
   | AnnDeclDotsTag      of annotated_decl dots
   | AnnFieldDotsTag     of annotated_field dots
+  | EnumDeclDotsTag     of enum_decl dots
   | DefParDotsTag       of define_param dots
   | TypeCTag            of typeC
   | ParamTag            of parameterTypeDef
@@ -858,6 +883,8 @@ val ident_of_expression_opt : expression -> ident option
  * [ident_of_expression_opt e] returns [Some id] iff [e] is of the form
  * [Ident e] *)
 
+val string_of_meta_name : meta_name -> string
+
 type 'a transformer = {
     baseType: (baseType -> string mcode list -> 'a) option;
     decimal: (string mcode -> string mcode -> expression ->
