@@ -1,5 +1,5 @@
 (*
- * This file is part of Coccinelle, lincensed under the terms of the GPL v2.
+ * This file is part of Coccinelle, licensed under the terms of the GPL v2.
  * See copyright.txt in the Coccinelle source code for more information.
  * The Coccinelle source code can be obtained at http://coccinelle.lip6.fr
  *)
@@ -21,7 +21,7 @@ let term s inif =
       | Ast.Exp(_) -> Past.Atomic ast
       | Ast.Decl(_) -> Past.Atomic ast
       | Ast.ReturnExpr(_,_,_) -> Past.Atomic ast
-      | Ast.MetaStmt(_,_,_,_) when inif -> Past.Atomic ast
+      | Ast.MetaStmt(_,_,_,_,_) when inif -> Past.Atomic ast
       | Ast.DisjRuleElem(_) -> Past.Atomic ast
       | _ -> fail())
   | _ -> fail()
